#include <Servo.h>

#define SCK 4      //   74HC595-pin 11
#define RCK 7      //   74HC595-pin 12
#define SIN 8      //   74HC595-pin 14

#define EN1 5          
#define EN2 6                      

#define IR_Senor_L   A1   //Define the infrared obstacle avoidance sensor pin on the left
#define IR_Senor_R   A0   //Define the infrared obstacle avoidance sensor pin on the right
#define Trig         A5   //Define the ultrasonic signal emitting interface
#define Echo         A4   //Define the ultrasonic signal receiving interface 
#define sound        A3   //Define buzzer pins

Servo myservo;
unsigned long cmd;

void setup() 
{
   // Serial port initialization
    Serial.begin(9600); 
   
    //Steering gear pin initialization
    myservo.attach(3); 

    //Set the pin mode
    pinMode(EN1, OUTPUT);
    pinMode(EN2, OUTPUT);
  
    pinMode(SCK, OUTPUT);
    pinMode(RCK, OUTPUT);
    pinMode(SIN, OUTPUT);
   
    pinMode(Trig,OUTPUT);   
    pinMode(Echo,INPUT);    
    pinMode(IR_Senor_L,INPUT);
    pinMode(IR_Senor_R,INPUT);

    pinMode(sound,OUTPUT);
    Motor(57,0,0);//stop
    BEEP();//Initialization complete, let the buzzer ring
}

void loop() 
{
   track();
}

void Motor(int Dir,int Speed1,int Speed2)
{
    analogWrite(EN1,Speed1); // Motor1 PWM speed regulation
    analogWrite(EN2,Speed2); // Motor2 PWM speed regulation

    digitalWrite(RCK,LOW); // Dir LATCH sets low level, writes direction of movement to prepare

    shiftOut(SIN,SCK,MSBFIRST,Dir);// Write the Dir direction of motion

    digitalWrite(RCK,HIGH);// Dir LATCH sets the high level to output the direction of motion
}


/*
*   Function: obtain ultrasonic sensor ranging data
*   The return value：dis
*/
float getDistance()
{
    float dis;  // Stores the variables of the measured data
    // The pin of the ultrasonic signal is pulled down
    digitalWrite(Trig, LOW);  
    // Keep low level at 2μs
    delayMicroseconds(2); 
    // The level of the pin of the ultrasonic signal is raised     
    digitalWrite(Trig, HIGH);  
    // Keep the high level at 10μs 
    delayMicroseconds(10);   
    // Keep the ultrasonic signal pin at low level   
    digitalWrite(Trig, LOW);    
    // Convert pulse time to distance (in cm)
    dis = pulseIn(Echo, HIGH) / 58.00;
    // Returns the measurement data in centimeters    
    return dis;
}

void track()
{
    int get_dis = 0;
    int pos;   // Steering gear Angle

    get_dis = getDistance();

    //If the sensor on the right encounters a follower, turn right
    if( digitalRead(IR_Senor_R) == 0)
    {
        myservo.write(60);   
        Motor(57,255,255);//turn right
        delay(300);
    }
    else if( digitalRead(IR_Senor_L) == 0)
    {    
        myservo.write(140);   
        Motor(198,255,255);
        delay(300);
    }
   // The distance is between 20cm and 40cm
    else if(get_dis > 20 && get_dis < 40)
    {
        myservo.write(100);
        Motor(106,255,255);;   
    }
    // Backup less than 15cm
    else  if(get_dis < 15 )
    {
        myservo.write(100);
        Motor(149,255,255);  
    } 
    else  
    {
       myservo.write(100);
       Motor(0,0,0);
    }  
}

void BEEP()
{   
    digitalWrite(sound, HIGH);
    delay(500);
    digitalWrite(sound, LOW); 
    delay(500);
}
