#include <Servo.h>
#include <IRremote.h>

#define FORWARD     0xFF02FD   
#define BACKWARD    0xFF9867   
#define TURNLEFT    0xFFE01F   
#define TURNRIGHT   0xFF906F   
#define STOP        0xFFA857   
#define Buzzer      0xFF7A85 
#define Avo_Obs     0xFF18E7
#define TRACK       0xFF30CF  

#define RECV_PIN  2    //Define infrared receiver pin
#define SCK 4      //   74HC595-pin 11
#define RCK 7      //   74HC595-pin 12
#define SIN 8      //   74HC595-pin 14
#define EN1 5          
#define EN2 6                      

#define IR_Senor_L   A1   //Define the infrared obstacle avoidance sensor pin on the left
#define IR_Senor_R   A0   //Define the infrared obstacle avoidance sensor pin on the right
#define Trig         A5   //Define the ultrasonic signal emitting interface
#define Echo         A4   //Define the ultrasonic signal receiving interface 
#define sound        A3   //Define buzzer pins

//Declare a variable type unique to the IrRemote library function (DECODE_RESULTS)
IRrecv irrecv(RECV_PIN); 
decode_results results; 
Servo myservo;
unsigned long cmd;

void setup() 
{
   // Serial port initialization
    Serial.begin(9600); 
    //enableIRIn
    irrecv.enableIRIn();
    //Steering gear pin initialization
    myservo.attach(3); 

    //Set the pin mode
    pinMode(EN1, OUTPUT);
    pinMode(EN2, OUTPUT);
  
    pinMode(SCK, OUTPUT);
    pinMode(RCK, OUTPUT);
    pinMode(SIN, OUTPUT);
   
    pinMode(Trig,OUTPUT);   
    pinMode(Echo,INPUT);    
    pinMode(IR_Senor_L,INPUT);
    pinMode(IR_Senor_R,INPUT);

    pinMode(sound,OUTPUT);
    Motor(57,0,0);//stop
    BEEP();//Initialization complete, let the buzzer ring
}

void loop() 
{
    HC06();
    run_car();
}

/*
*   function name：HC06
*   Function: Receive wireless remote control command
*/
void HC06()
{
   // Check whether infrared remote control signal is received
    if (irrecv.decode(&results) ) 
    { 
        //The received value is stored in res_val
        unsigned long res_val = results.value;  
        
        Serial.print("res_val=");
        Serial.println(res_val,HEX);//Output instruction information

        irrecv.resume();//Waiting for the next reception

        switch(res_val)
        {
            case Buzzer:
            cmd = Buzzer;
            break;

            case Avo_Obs: 
            cmd = Avo_Obs;
            break;

            case TRACK: 
            cmd = TRACK;
            break;

            case TURNLEFT:
            cmd = TURNLEFT;
            break;

            case TURNRIGHT: 
            cmd = TURNRIGHT;
            break;

            case FORWARD: 
            cmd = FORWARD;
            break;

            case BACKWARD: 
            cmd = BACKWARD;
            break;

            case STOP: 
            cmd = STOP;
            break;
        }
    }  
}

/*  
*   function name：run_car
*   Function: car function selection 
*/
void run_car()
{ 
    //cmd：Function selection parameter
    switch(cmd)
    {
         case FORWARD:
        Motor(106,255,255);
        Serial.println("go forward!");
        break;

        case BACKWARD:   
        Motor(149,255,255);
        Serial.println("go back!");
        break;

        case TURNLEFT:
        Motor(198,255,255);
        Serial.println("go left!");  
        break;

        case TURNRIGHT: 
        Motor(57,255,255);
        Serial.println("go right!");
        break;

        case Buzzer: 
        BEEP(); 
        Serial.println("go blinks!");
        break;

        case STOP:
        Motor(0,0,0);
        Serial.println("go stop!");
        break;

        case Avo_Obs:
        avoidance(15);
        Serial.println("go AvoidObstacle!");
        break;

        case TRACK: 
        track();
        Serial.println("go track!");
        break;
    }
}

/* function name: Motor();
 
* Function: change the moving direction and speed of the car through the entry parameters
* Entry parameter 1: movement direction of DRI car
* Input parameters 2~3: Speed1~Speed2 motor speed, value range 0~255
* DRI value description (forward :106;
Back: 149;
Left: 198;
Right: 57.
Stop: 0;
* Return value: None
*/
void Motor(int Dir,int Speed1,int Speed2)
{
    analogWrite(EN1,Speed1); //Motor1 PWM speed regulation
    analogWrite(EN2,Speed2); //Motor2 PWM speed regulation

    //DIR_LATCH sets the low level and writes the direction of movement in preparation
    digitalWrite(RCK,LOW); 
    // Write the Dir direction of motion
    shiftOut(SIN,SCK,MSBFIRST,Dir);
    //DIR_LATCH sets the high level and outputs the direction of movement
    digitalWrite(RCK,HIGH);
}

/*
* Function: Obstacle avoidance
* Parameter: set_dis sets the obstacle avoidance distance
*/
void avoidance(int set_dis)
{
    int pos;
    int dis[3];// distance
  
    Motor(106,255,255);

    myservo.write(100); //Steering engine back to center
    //Obtain the distance between the cart and the obstacle and store it in dis[1]
    dis[1] = getDistance(); 

    if(dis[1] < set_dis || digitalRead(IR_Senor_R) == 0 ||
                           digitalRead(IR_Senor_L) == 0)
    {
        Motor(0,0,0);//Stop the car

        //If the left and right infrared obstacle avoidance sensors do not encounter obstacles when the steering gear rotates
        if(digitalRead(IR_Senor_R) == 1 && digitalRead(IR_Senor_L) == 1)  
        {
            for (pos = 100; pos <= 140; pos += 1) 
            {
                myservo.write(pos); 
                delay(5);         
            }
           // Measure the distance between the left obstacle and the cart, and store the measurement data in dis[2]
            dis[2]=getDistance(); 

            for (pos = 140; pos >= 60; pos -= 1) 
            {
                myservo.write(pos);   
                delay(5);             
                if(pos == 100)  
                {   // Stores measurement data in dis[1]
                    dis[1] = getDistance(); 
                } 
            }
           // Record the range data on the right side of the trolley
            dis[0] = getDistance(); 
            for (pos = 60; pos <= 100; pos += 1) 
            {
                myservo.write(pos);
                delay(5); 
            }
        }

       
        if(dis[0] < dis[2] || digitalRead(IR_Senor_R) == 0) 
        {
            Motor(149,255,255);// back
            delay(200);
            Motor(198,255,255);//left
            delay(300);
        }
        //The right is more distant from the obstacle than the left
        else if (dis[0] > dis[2] || digitalRead(IR_Senor_L) == 0)
        {
            Motor(149,255,255);

            delay(200);
            Motor(57,255,255);
            delay(300);
        } 
    }
}

/*
* Function name: getDistance
* Function: obtain ultrasonic sensor ranging data
* Return value: dis
*/
float getDistance()
{
    float dis;  
    
    digitalWrite(Trig, LOW);    
    delayMicroseconds(2);      
    digitalWrite(Trig, HIGH);   
    delayMicroseconds(10);      
    digitalWrite(Trig, LOW);    

    dis = pulseIn(Echo, HIGH) / 58.00;
        
    return dis;
}


/* 
*  Function: Car follow
*/
void track()
{
    int get_dis = 0;
    int pos;   // Steering gear Angle

    get_dis = getDistance();

    //If the sensor on the right encounters a follower, turn right
    if( digitalRead(IR_Senor_R) == 0)
    {
        myservo.write(60);   
         Motor(57,255,255);//turn right
        delay(300);
    }
    else if( digitalRead(IR_Senor_L) == 0)
    {    
        myservo.write(140);   
        Motor(198,255,255);
        delay(300);
    }
   // The distance is between 20cm and 40cm
    else if(get_dis > 20 && get_dis < 40)
    {
        myservo.write(100);
         Motor(106,255,255);;   
    }
    // Backup less than 15cm
    else  if(get_dis < 15 )
    {
        myservo.write(100);
        Motor(149,255,255);  
    } 
    else  
    {
        myservo.write(100);
       Motor(0,0,0);
    }  
}

void BEEP()
{   
    digitalWrite(sound, HIGH);
    delay(500);
    digitalWrite(sound, LOW); 
    delay(500);
}
